
;*************************
;*  Chapter 4 Link Code  *
;*************************

; Last edit 17-Oct-98

; High RAM Usage
SpriteDMA       EQU     $ff80
StackPtr        EQU     $fffe

        INCLUDE "hardware.inc"
        INCLUDE "ibmpc1.inc"
        INCLUDE "load1.inc"

;Version 1.2 or later of 'hardware.inc' is required.
        rev_Check_hardware_inc 1.5
;Version 1.0 or later of 'load1.inc' is required.
        rev_Check_load1_inc 1.0

        SECTION "Low Ram",BSS

; * SpriteTable *
SpriteTab       DS      40*4
VBlankDone      DS      1
pressed         DS      1
Counter         DS      1

        SECTION "Org $0",HOME
        ret


        SECTION "Org $8",HOME[$8]
        ret

        SECTION "Org $40",HOME[$40]
        jp      VBlankIntr

        SECTION "Org $50",HOME[$50]
        reti

        SECTION "Org $58",HOME[$58]

        jp      SIOIntr

; DMA copy sprite data to sprite ram

VBlankFirst:
        ld      a,$c0 ;SpriteTable/256
        ldh     [rDMA],a
        ld      a,$28
VBlnkW: dec     a
        jr      nz,VBlnkW
        ret
VBlankLast:


VBlankIntr:
        push    af
        push    bc
        push    de
        push    hl

        call    SpriteDMA

; Put a state machine for updating 1 of 4
; sets of video data during this VBlank Interrupt.

        ld      a,1
        ld      [VBlankDone],a

        pop     hl
        pop     de
        pop     bc
        pop     af
        reti


        DB      "Date 981017"          ; Date stamping info for
                                       ; reference use only
        SECTION "Org $100",HOME[$100]

;*** Beginning of rom execution point ***

        nop
        jp      ColdBoot

        NINTENDO_LOGO                   ; Nintendo graphic logo

;Rom Header Info

;    0123456789ABCDEF
 DB "CHAPTER 4 LINK",0       ; Cart name   15 bytes
 DB $0                       ; $00 = GB/SGB, $80 = CGB
 DB $36,$30
 DB 0                         ; SGB = 3
 DB CART_ROM_MBC1_RAM_BAT   ;$19               ; Cart type
 DB CART_ROM_256K             ; ROM Size (in bits)
 DB CART_RAM_NONE             ; RAM Size (in bits)
 DB 1                         ; Country code
 DB $33                       ; Maker ID (Old)
 DB 0                         ; Version     =0
 DB $e2                       ; Complement check (important)
 DW $c40e                     ; Checksum (not important)

; Macros

        INCLUDE "ibmpc1.inc"

; Library routines

        INCLUDE "memory1.asm"   ; memory set/copy
        INCLUDE "keypad.asm"    ; keypad
        INCLUDE "comm.asm"      ; serial i/o stuff

        SECTION "Main Code",HOME

ColdBoot:
        di
        ld      sp,StackPtr                ; Initialize stack pointer

; *** Initialize screen stuff ***

        call    Initialize              ; Init screen crap

; *** Display 'Press Start' message ***

        call    DisplayPressStart

; *** Screen on ***

        call    ScreenOn

; *** Initialize comm stuff ***

        call    comm_not_ready

        ei

; *** Wait for start to be pressed ***

        xor     a
        ld      [pressed],a

WaitForStart:
        ld      a,[pressed]
        or      a
        jr      nz,.skip

        call    pad_Read
        or      a                       ; Any button pressed?
        jr      z,.skip                 ; no

        call    comm_start_master_game

        ld      a,1
        ld      [pressed],a
.skip:
        call    comm_check_other_status
        or      a                       ; Is other unit ready?
        jr      nz,StartGame            ; yes, begin

        call    WaitForVBlank

        jr      WaitForStart


; *** Start game ***

StartGame:
        call    DisplayGameBegin

MainLoop:

        call    pad_Read
        ld      [td],a

        call    comm_transfer           ; Transmit 'td', Receive 'rd'

        call    DrawButtons             ; Display 'rd' as buttons
        call    MoveSprite

; Do everything needed for one frame of video

;  For the purposes of sending data to video ram,
; do that in the VBlank interrupt routine.
; Syncronize is not necessarily synchronized
; with the VBlank of this system so don't use it
; for transfers to VRAM!!!

        call    Synchronize

        jr      MainLoop



DrawButtons:
        ld      a,[rd]

        ld      b,a
        ld      hl,AddrList
        ld      d,8
.loop:
        rlc     b
        ld      a,b
        and     1

        call    DrawButton

        dec     d
        jr      nz,.loop
        ret

DrawButton:
        push    bc
        push    de
        ld      e,[hl]
        inc     hl
        ld      d,[hl]
        inc     hl
        push    hl

        ld      b,32
        or      a
        jr      z,.wait
        ld      b,0
.wait:
        ldh     a,[$41]
        and     2
        jr      nz,.wait

        ld      a,b
        ld      [de],a

        pop     hl
        pop     de
        pop     bc
        ret

AddrList:
        dw      $9800+5+(32*10)  ;down
        dw      $9800+5+(32*8)   ;up
        dw      $9800+4+(32*9)   ;left
        dw      $9800+6+(32*9)   ;right
        dw      $9800+12+(32*13) ;start
        dw      $9800+9+(32*13)  ;select
        dw      $9800+14+(32*9)  ;B
        dw      $9800+17+(32*8)  ;A

MoveSprite:
        ld      a,[Counter]
        inc     a
        and     $7
        ld      [Counter],a
        ret     nz

        ld      a,[SpriteTab+1]
        inc     a
        and     $7f
        ld      [SpriteTab+1],a
        ret

; Wait for VBlank to occur

WaitForVBlank:
        ld      a,[VBlankDone]
        or      a
        jr      z,WaitForVBlank

        xor     a
        ld      [VBlankDone],a
        ret

; Init video stuff

Initialize:
        call    ScreenOff

        ld      hl,VBlankFirst
        ld      de,SpriteDMA
        ld      bc,VBlankLast-VBlankFirst
        call    mem_Copy

        ld      hl,SpriteTab
        ld      bc,4*40
        xor     a
        call    mem_Set                 ; Clear sprite table

        ld      hl,$fe00
        ld      bc,4*40
        xor     a
        call    mem_Set                 ; Clear sprite mem

        ld      a,16+4
        ld      [SpriteTab+0],a
        ld      a,8
        ld      [SpriteTab+1],a
        ld      a,1
        ld      [SpriteTab+2],a
        ld      a,0
        ld      [SpriteTab+3],a         ; Put a sprite in upper left corner

        ld      a,IEF_SERIAL + IEF_VBLANK
        ldh     [rIE],a                 ; Enable VBlank & Serial Int interrupt

        xor     a
        ldh     [rSCX],a
        ldh     [rSCY],a

        ld      a,$e4
        ldh     [rBGP],a
        ldh     [rOBP0],a
        ldh     [rOBP1],a

        ld      hl,CharSet
        ld      de,_VRAM
        ld      bc,256*8
        call    mem_CopyMono            ; Load font

        ld      hl,_SCRN0
        ld      bc,32*32
        ld      a,32
        call    mem_Set                 ; Clear screen
        ret

; *** Turn screen on ***

ScreenOn:
        ld      a,LCDCF_ON|LCDCF_BG8000|LCDCF_BG9800|LCDCF_BGON|LCDCF_OBJ8|LCDCF_OBJON
        ldh     [rLCDC],a
        ret

; Turn off the LCD

ScreenOff:
        ldh     a,[rLCDC]
        bit     7,a             ; Is LCD already off?
        ret     z               ; yes, exit

        ldh     a,[rIE]
        push    af
        res     0,a
        ldh     [rIE],a

.wait:  ldh     a,[rLY]
        cp      145
        jr      nz,.wait

        ldh     a,[rLCDC]
	res	7,a
        ldh     [rLCDC],a

        pop     af
        ldh     [rIE],a
        ret

; *** Display 'Press Start' message ***

DisplayPressStart:
        ld      hl,PressStart
        ld      de,_SCRN0+4+(32*3)
        ld      bc,11
        call    mem_Copy

        call    DrawOutlines
        ret

; *** Draw outline of virtual buttons ***

DrawOutlines:
        ld      hl,Outlines
        ld      de,_SCRN0+(32*7)
        ld      bc,8*32
        call    mem_CopyVRAM
        ret

; *** Display 'Game Begin' message ***

DisplayGameBegin:
        ld      hl,GameBegin
        ld      de,_SCRN0 + 4 + (32*5)
        ld      bc,10
        call    mem_CopyVRAM            ; Display 'Game Begin' msg

        ret

GameBegin:
        DB      "Game Begin"

PressStart:
        DB      "Press Start"

; *** Virtual buttons map ***

Outlines:
        DB      "    ",$c9,$cd,$bb,"         ",$c9,$cd,$bb," xxxxxxxxxxxx"
        DB      "   ",$c9,$bc,32,$c8,$bb,"     ",$c9,$cd,$bb,$ba,32,$ba," xxxxxxxxxxxx"
        DB      "   ",$ba,"   ",$ba,"     ",$ba,32,$ba,$c8,$cd,$bc," xxxxxxxxxxxx"
        DB      "   ",$c8,$bb,32,$c9,$bc,"     ",$c8,$cd,$bc,"    xxxxxxxxxxxx"
        DB      "    ",$c8,$cd,$bc,"             xxxxxxxxxxxx"
        DB      "        ",$c9,$cd,$bb,$c9,$cd,$bb,"      xxxxxxxxxxxx"
        DB      "        ",$ba,$20,$ba,$ba,$20,$ba,"      xxxxxxxxxxxx"
        DB      "        ",$c8,$cd,$bc,$c8,$cd,$bc,"      xxxxxxxxxxxx"

CharSet:
        chr_IBMPC1      1,8

